#include "level.h"
#include "graphics.h"

#include<stdlib.h>

Clevel::Clevel(int w, int h, int bMax, int enemies, DATAFILE *dataFile)
{
    width = w;
    height = h;
    srand(time(NULL));
    blocks.resize(h);
    for(vector<vector<int> >::iterator i = blocks.begin(); i != blocks.end(); i++)
        (*i).resize(w);
    //add blocks in....
    for(int i = 0; i < bMax; i++)
    {
        //add a block
        setLoc(rand() % w, rand() %h, 1);
    }
    
    //set edges to solid
    for(int i = 0; i < w; i++)
    {
        setLoc(i, 0, 1);
        setLoc(i, h - 1, 1);
    }
    for(int i = 0; i < h; i++)
    {
        setLoc(0, i, 1);
        setLoc(w - 1, i, 1);
    }

    for(int x = 1; x < w - 1; x++)
    {
        for(int y = 1; y < h - 1; y++)
        {
            if(getLoc(x, y) != 0) //we should do some work....
            {
                if(getLoc(x + 1, y) != 0 && getLoc(x - 1, y) != 0)
                    setLoc(x, y, 3);
                else if(getLoc(x + 1, y) != 0 && getLoc(x - 1, y) == 0)
                    setLoc(x, y, 2);
                else if(getLoc(x + 1, y) == 0 && getLoc(x - 1, y) == 0)
                    setLoc(x, y, 5);
                else
                    setLoc(x, y, 4);
            }
        }
    }
            
    //load bitmaps from datafile
    blockBitmaps.push_back(create_bitmap(32, 32));
    clear_to_color(blockBitmaps[0], makecol(255, 0, 255)); //transparent
    for(int n = 0; n < 5; n++) //load the other 4 bitmaps
        blockBitmaps.push_back((BITMAP *)dataFile[n].dat);
}

Clevel::~Clevel()
{

}

int Clevel::blocksOnRow(int r)
{
    int ret = 0;
    for(vector<int>::iterator i = blocks[r].begin(); i != blocks[r].end(); i++)
    {
        if((*i) != 0)
            ret++;
    }
    return ret;
}

void Clevel::draw()
{
    CGraphics::instance()->offX = (800 - width * 32) / 2;
    CGraphics::instance()->offY = (600 - height * 32) / 2;
    for(int x = 0; x < width; x++)
    {
        for(int y = 0; y < height; y++)
        {
            draw_sprite(CGraphics::instance()->getDB(), blockBitmaps[getLoc(x, y)], x * 32 + CGraphics::instance()->offX, y * 32 + CGraphics::instance()->offY);
        }
    }
}

bool Clevel::checkCollision(int x, int y, int w, int h)
{
    for(int bX = 0; bX < width; bX++)
    {
        for(int bY = 0; bY < height; bY++)
        {
            if(getLoc(bX, bY) == 1) //big block
            {
                if(!(x + w < bX * 32 || bX * 32 + 32 < x || y + h < bY * 32 || bY * 32 + 32 < y))
                {
                    return true;
                }
            }
            else if(getLoc(bX, bY) != 0) //smaller block
            {
                if(!(x + w < bX * 32 || bX * 32 + 32 < x || y + h < bY * 32 || bY * 32 + 16 < y))
                {
                    return true;
                }
            }
        }
    }
    return false;
}

void Clevel::setToStart(CPlayer *playa)
{
    for(int y = height - 2; y > 0; y--)
    {
        for(int x = 1; x < width; x++)
        {
            if((getLoc(x, y) == 0)&&(getLoc(x, y + 1) == 0)) //empty space
            {
                playa->setX(x);
                playa->setY(y);
                return;
            }
        }
    }
}
