#include "player.h"
#include "graphics.h"



void CPlayer::setX(int to)
{
    int animation = currAction;
    if(!facingRight)
        animation++;
    x = to * 32 + (animations[animation]->getCurr()->w)/2;
}

void CPlayer::setY(int to)
{
    int animation = currAction;
    if(!facingRight)
        animation++;
    y = to * 32 + (animations[animation]->getCurr()->h)/2;
}

CPlayer::CPlayer(DATAFILE *theFile)
{
    DATAFILE *temp;
   //we assume that the file is in working shape and the correct length and all that
   animations.push_back(new CAnimation()); //stand right
   animations.back()->addFrame(((BITMAP *)((DATAFILE *)theFile[6].dat)[0].dat), 3600); //add the frame
   animations.push_back(new CAnimation()); //stand left
   animations.back()->addFrame(((BITMAP *)((DATAFILE *)theFile[6].dat)[1].dat), 3600); //add the frame
   animations.push_back(new CAnimation()); //wall right
   animations.back()->addFrame(((BITMAP *)((DATAFILE *)theFile[5].dat)[0].dat), 3600); //add the frame
   animations.push_back(new CAnimation()); //wall left
   animations.back()->addFrame(((BITMAP *)((DATAFILE *)theFile[5].dat)[1].dat), 3600); //add the frame
   animations.push_back(new CAnimation((DATAFILE *)theFile[4].dat, 9)); //turn from left to right
   animations.push_back(new CAnimationR((DATAFILE *)theFile[4].dat, 9)); //turn from right to left
   animations.push_back(new CAnimation((DATAFILE *)theFile[3].dat, 30)); //jump right
   animations.push_back(new CAnimation((DATAFILE *)theFile[2].dat, 30)); //jump left
   animations.push_back(new CAnimation((DATAFILE *)theFile[1].dat, 40)); //walk right
   animations.push_back(new CAnimation((DATAFILE *)theFile[0].dat, 40)); //walk left
   facingRight = true;
   x = 100.0f;
   y = 100.0f;
   velocityX = 0.0f;
   velocityY = 0.0f;
   maxXV = 600.0f;
   maxYV = 600.0f;
   groundAccel = 30.0f;
   airAccel = 8.0f;
   groundFriction = 0.09f;
   airFriction = 0.02f;
   jumpSpeed = -35.0f * 9.82f;
   currAction = PLAYER_RUN;
   nonLoopAnimationTime = -1;
   currNonTime = 0;
   onGround = false;
   onRightWall = 0;
   onLeftWall = 0;
   leftHold = 0;
   rightHold = 0;
   life = 100;
   lifeRegenCounter = 0;
}

CPlayer::~CPlayer()
{
    for(vector<CAnimation *>::iterator i = animations.begin(); i != animations.end(); i++)
        delete *i;
}

bool CPlayer::checkCollision(int wx, int wy, int ww, int wh)
{
    int animation = currAction;
    if(!facingRight)
        animation++;
        
    int cx = (int)x - (animations[animation]->getCurr()->w / 2) + 7;
    int cy = (int)y - (animations[animation]->getCurr()->h / 2) + 5;
    int cw = (animations[animation]->getCurr()->w) - 14;
    int ch = (animations[animation]->getCurr()->h) - 10;
    return !(wx + ww < cx || cx + cw < wx || wy + wh < cy || cy + cw < wy);
}

void CPlayer::draw()
{
    int animation = currAction;
    if(!facingRight)
        animation++;
    int dX = (int)x;
    int dY = (int)y;
    dX -= (animations[animation]->getCurr()->w)/2; //x refers to the center
    dY -= (animations[animation]->getCurr()->h)/2; //y too
    draw_sprite(CGraphics::instance()->getDB(), animations[animation]->getCurr(), dX + CGraphics::instance()->offX, dY + CGraphics::instance()->offY);
    /*if(onGround)
        textprintf_ex(CGraphics::instance()->getDB(), font, 0, 0, makecol(255, 255, 255), makecol(0, 0, 0), "On Ground");
    if(onRightWall)
        textprintf_ex(CGraphics::instance()->getDB(), font, 0, 20, makecol(255, 255, 255), makecol(0, 0, 0), "On Right Wall");
    if(onLeftWall)
        textprintf_ex(CGraphics::instance()->getDB(), font, 0, 20, makecol(255, 255, 255), makecol(0, 0, 0), "On Left Wall");
    */
}


void CPlayer::update(int input, Clevel *theMap)
{
    lifeRegenCounter++;
    if(lifeRegenCounter >= 90)
    {
        life += 1;
        if(life > 100)
            life = 100;
        lifeRegenCounter = 0;
    }
    
    //do animation and input stuff
    int animation = currAction;
    if(!facingRight)
        animation++;
    animations[animation]->update();
    currNonTime++;
    if(currNonTime == nonLoopAnimationTime)
    {
        setPlayerAction(PLAYER_RUN, facingRight);
        nonLoopAnimationTime = -1;
    }
    bool left = false;
    bool right = false;
    bool jump = false;
    if(input >= 4)
    {
        jump = true;
        input -= 4;
    }
    if(input >= 2)
    {
        right = true;
        input -= 2;
    }
    if(input == 1)
        left = true;
    if((left)&&(onGround))
    {
        setPlayerAction(PLAYER_RUN, false);
        velocityX -= groundAccel; 
    }
    if((right)&&(onGround))
    {
        setPlayerAction(PLAYER_RUN, true);
        velocityX += groundAccel;
    }
    if((jump)&&(onGround))
    {
        setPlayerAction(PLAYER_JUMP, facingRight);
        velocityY += jumpSpeed;
    }

    if((right)&&(onLeftWall))
    {
        leftHold = 10;
        facingRight = false;
        setPlayerAction(PLAYER_WALL_HOLD, facingRight);
        onLeftWall = 0;
        waitingForLeftHoldTimeout = true;
    }
    if((left)&&(onRightWall))
    {
        rightHold = 10;
        facingRight = true;
        setPlayerAction(PLAYER_WALL_HOLD, facingRight);
        onRightWall = 0;
        waitingForRightHoldTimeout = true;
    }
    
    if((rightHold)&&(jump))
    {
        velocityY = jumpSpeed;
        velocityX += jumpSpeed / 4;
        setPlayerAction(PLAYER_JUMP, false);
        rightHold = 0;
    }
    
    if((leftHold)&&(jump))
    {
        velocityY = jumpSpeed;
        velocityX -= jumpSpeed / 4;
        setPlayerAction(PLAYER_JUMP, true);
        leftHold = 0;
    }

    if((!onGround)&&(left))
    {
        velocityX -= airAccel;
    }
    
    if((!onGround)&&(right))
    {
        velocityX += airAccel;
    }
    
    velocityY += 9.82; //gravity
    velocityY -= airFriction * velocityY;
    
    if(onGround)
        velocityX -= groundFriction * velocityX;
    else
        velocityX -= airFriction * velocityX;
    if(velocityX > maxXV)
        velocityX = maxXV;
    else if(velocityX < -maxXV)
        velocityX = -maxXV;
        
    if(velocityY > maxYV)
        velocityY = maxYV;
    else if(velocityY < -maxYV)
        velocityY = -maxYV;
    //update location, check collision, update again and so on.
    x += velocityX/60;
    if(onLeftWall)
        onLeftWall--;
    if(onRightWall)
        onRightWall--;
    leftHold--;
    rightHold--;
    if(leftHold < 0)
    {
        leftHold = 0;
        if(waitingForLeftHoldTimeout)
            setPlayerAction(PLAYER_JUMP, facingRight);
        waitingForLeftHoldTimeout = false;
    }
    if(rightHold < 0)
    {
        if(waitingForRightHoldTimeout)
            setPlayerAction(PLAYER_JUMP, facingRight);
        rightHold = 0;
        waitingForRightHoldTimeout = false;
    }
    if(theMap->checkCollision((int)x - (animations[animation]->getCurr()->w / 2) + 7, (int)y - (animations[animation]->getCurr()->h / 2) + 6, animations[animation]->getCurr()->w - 14, animations[animation]->getCurr()->h - 12))
    {
        if(velocityX > 0)
            onRightWall = 6;
        else if(velocityX < 0)
            onLeftWall = 6;
        if(facingRight) //moving right
        {
            int blockLocation = (int)x/32;
            x = blockLocation * 32 + 32 - animations[animation]->getCurr()->w /2 + 7;
            velocityX = 0.0f;
        }
        else if(!facingRight) //left
        {
            int blockLocation = (int)x/32;
            x = blockLocation * 32 + animations[animation]->getCurr()->w /2 - 7;
            velocityX = 0.0f;
        }
    }
    float oldY = y;
    y += velocityY/60;
    onGround = false;    
    if(theMap->checkCollision((int)x - (animations[animation]->getCurr()->w / 2) + 8, (int)y - (animations[animation]->getCurr()->h / 2) + 5, animations[animation]->getCurr()->w - 16, animations[animation]->getCurr()->h - 10))
    {
        if(velocityY > 0) //moving down
        {
            y = oldY;
            velocityY = 0.0f;
            onGround = true;
            //move straight onto the ground
            while(!theMap->checkCollision((int)x - (animations[animation]->getCurr()->w / 2) + 8, (int)y - (animations[animation]->getCurr()->h / 2) + 5, animations[animation]->getCurr()->w - 16, animations[animation]->getCurr()->h - 10))
                y += 1.0f;
        }
        else if(velocityY < 0) //up
        {
            y = oldY;
            velocityY = 0.0f;
        }
    }
    
    if((onGround)&&(currAction != PLAYER_TURN))
    {
        if(velocityX > 0)
        {
            if(velocityX < 15.0f)
            {
                setPlayerAction(PLAYER_STAND, facingRight);
            }
            else
            {
                setPlayerAction(PLAYER_RUN, facingRight);
            }
        }
        else
        {
            if(velocityX > -15.0f)
            {
                setPlayerAction(PLAYER_STAND, facingRight);
            }
            else
            {
                setPlayerAction(PLAYER_RUN, facingRight);
            }
        }
    }
    
    
}

void CPlayer::setPlayerAction(int to, bool right)
{
    if(((to == -1)&&(right != facingRight))||(to == PLAYER_RUN)&&(facingRight != right))
    {
        //dir change only
        if((currAction == PLAYER_RUN)||(currAction == PLAYER_STAND)) //must turn
        {
            facingRight = right;
            currAction = PLAYER_TURN;
            nonLoopAnimationTime = 9;
            int animation = currAction;
            if(!facingRight)
            {
                animation++;
            }
            animations[animation]->reset();
            currNonTime = 0;
        }
        else
        {
            facingRight = right; //otherwise we really don't care
        }
    }
    else if(to != -1)
    {
        currAction = to;
        facingRight = right;
    }
}
